package org.sc3d.apt.julia.v6;

import org.sc3d.apt.screen.v4.Screen;

public class BitmapJulia {

  public static void main(String[] args) {
    if (args.length!=2) throw new IllegalArgumentException(
      "Syntax java org.sc3d.apt.julia.v4.Main <size> <skip>"
    );
    final int SIZE = Integer.parseInt(args[0]);
    final int SKIP = Integer.parseInt(args[1]);
    final int SIZE2 = SIZE*SIZE;
    final int HALF = (SIZE2+1)/2;
    final Screen SCREEN = new Screen(SIZE, SIZE, "Julia sets");
    final int[] PIXELS = SCREEN.getPixels();
    final int[] DEST = new int[HALF];
    final int[] COUNT = new int[HALF];
    final double SCALE = 4.0/SIZE, ISCALE = SIZE/4.0;
    Thread.currentThread().setPriority(Thread.MIN_PRIORITY);
    while (true) {
      // Change the constant.
      double cx = (SCREEN.getMouseX()+0.5)*SCALE-2.0;
      double cy = (SCREEN.getMouseY()+0.5)*SCALE-2.0;
      // Set up the equation.
      for (int v=0; v*2<SIZE; v++) {
        for (int u=0; u<SIZE; u++) {
          double x = (u+0.5)*SCALE-2.0;
          double y = (v+0.5)*SCALE-2.0;
          int dest = -1, count = 0;
          do {
            if (count>=SKIP) {
              dest = (int)(
                Math.floor((x+2.0)*ISCALE) +
                Math.floor((y+2.0)*ISCALE)*SIZE
              );
              if (dest>=HALF) dest = SIZE2+~dest;
              break;
            }
            double o = x; x = (x+y)*(x-y) + cx; y = 2*o*y + cy;
            count++;
          } while (x>=-2 && x<2 && y>=-2 && y<2);
          DEST[u+SIZE*v] = dest;
          COUNT[u+SIZE*v] = count;
        }
      }
      // Solve it.
      for (int i=0; i<HALF; i++) {
        while (DEST[i]!=-1 && DEST[i]!=i) {
          for (int j=i; j!=-1; j=DEST[j]) {
            int k = DEST[j];
            if (k==-1) break;
            int l = DEST[k];
            if (k==l) { DEST[j] = j; break; }
            DEST[j] = l;
            COUNT[j] += COUNT[k];
          }
        }
        PIXELS[i] = PIXELS[SIZE2+~i] =
          DEST[i]==i ? 0 : Screen.sequenceColour(COUNT[i]);
      }
      // Display a frame.
      SCREEN.doFrame();
    }
  }
}
