package org.sc3d.apt.julia.v6;

import java.awt.event.*;

/** A pair of ArgandDiagrams, one of which shows the square of what the other shows. Drawing with the mouse on either diagram updates both. */
public class FunctionDemo implements MouseListener, MouseMotionListener {
  /** Constructs a FunctionDemo.
   * @param input an ArgandDiagram that will show the square-root of 'output'.
   * @param output an ArgandDiagram that will show the square of 'input'.
   */
  public FunctionDemo(ArgandDiagram input, ArgandDiagram output) {
    this.input = input; this.output = output;
    this.colour = 0;
    this.input.addMouseListener(this);
    this.output.addMouseListener(this);
  }
  
  /* New API. */
  
  /** One of the ArgandDiagrams passed to the constructor. */
  public final ArgandDiagram input, output;
  
  /* Implement things in MouseListener. */
  
  public void mouseClicked(MouseEvent e) {}
  public void mouseEntered(MouseEvent e) {}
  public void mouseExited(MouseEvent e) {}
  
  public void mousePressed(MouseEvent e) {
    this.colour++;
    this.input.addMouseMotionListener(this);
    this.output.addMouseMotionListener(this);
  }
  public void mouseReleased(MouseEvent e) {
    this.input.removeMouseMotionListener(this);
    this.output.removeMouseMotionListener(this);
  }

  /* Implement things in MouseMotionListener. */

  public void mouseMoved(MouseEvent e) {}

  public void mouseDragged(MouseEvent e) {
    Complex inZ, outZ;
    if (e.getSource()==this.input) {
      inZ = this.input.pixelToComplex(e.getX(), e.getY());
      outZ = inZ.sqr();
    } else {
      outZ = this.input.pixelToComplex(e.getX(), e.getY());
      inZ = outZ.sqrt();
    }
    this.input.drawFatPoint(inZ, 2, this.colour);
    this.input.drawFatPoint(inZ.neg(), 2, this.colour);
    this.output.drawFatPoint(outZ, 2, this.colour);
  }
  
  /* Private. */
  
  private int colour;
  
  /* Test code. */
  
  public static void main(String[] args) {
    final FunctionDemo me = new FunctionDemo(
      ArgandDiagram.makeStandardDiagram(512),
      ArgandDiagram.makeStandardDiagram(512)
    );
    me.input.display("Square-root");
    me.output.display("Square");
  }
}
