package org.sc3d.apt.julia.v6;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;

/** A Canvas for displaying a BufferedImage. */
public class ImageCanvas extends Canvas {
  /** Constructs an ImageCanvas. If the supplied BufferedImage is not the same size as the ImageCanvas, it will be scaled to fit. */
  public ImageCanvas(BufferedImage image, int width, int height) {
    this.image = image;
    this.width = width; this.height = height;
    this.setSize(width, height);
  }
  
  /* New API. */
  
  /** The width of this ImageCanvas in pixels. */
  public final int width;
  
  /** The height of this ImageCanvas in pixels. */
  public final int height;
  
  /** The BufferedImage displayed on this ImageCanvas. If you scribble on the Image, you should then call 'repaint()' or 'paint(getGraphics())'. */
  public final BufferedImage image;
  
  /** Displays this ImageCanvas in its own window. Closing the window calls 'System.exit(0)'. */
  public void display(String title) {
    final Frame frame = new Frame(title);
    frame.addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) { System.exit(0); }
    });
    frame.add(this);
    frame.pack();
    frame.show();
  }
  
  /* Override things in Component. */
  
  /** Draws 'image'. */
  public void paint(Graphics g) {
    g.drawImage(this.image, 0, 0, this.width, this.height, null);
  }
  
  /** Calls 'paint(g)'. */
  public void update(Graphics g) {
    this.paint(g);
  }
}
